from hotbox.utils import _set_templates_home

NAME = "hotbox"
DESC = "🚀 Run your code 📦 on Firecracker MicroVMs 🔥 in the cloud ☁️"
API_V0 = "/api/v0"
SERVER_MODULE_NAME = "hotbox.api:api"

TEMPLATES_HOME = _set_templates_home()
DEFAULT_IMAGE_TEMPLATE_DIR = f"{TEMPLATES_HOME}/image"
DEFAULT_LANG_TEMPLATE_DIR = f"{TEMPLATES_HOME}/lang"
DEFAULT_RUN_APP_TEMPLATE_FILEPATH = f"{TEMPLATES_HOME}/run_app.sh.j2"
DEFAULT_USERDATA_TEMPLATE_FILEPATH = f"{TEMPLATES_HOME}/ec2_userdata.sh.j2"



import os
from datetime import datetime
from typing import List, Optional

from fastapi import APIRouter, BackgroundTasks, FastAPI, File, Form, Query, UploadFile
from fastapi.responses import JSONResponse
from pydantic import Json

from hotbox import __version__
from hotbox.app import app_svc
from hotbox.const import API_V0, DESC, NAME
from hotbox.types import (
    CreateAppRequest,
    DeleteAppsResponse,
    GetAppsResponse,
    HealthcheckResponse,
)

os.environ["TZ"] = "UTC"

api = FastAPI(
    title=NAME,
    desc=DESC,
    version=__version__,
)

health_router = APIRouter(
    prefix=API_V0,
    tags=["health"],
)

app_router = APIRouter(
    prefix=API_V0,
    tags=["app"],
)


@health_router.get(
    "/healthcheck",
    response_model=HealthcheckResponse,
)
async def healthcheck() -> HealthcheckResponse:
    return HealthcheckResponse(
        message="ok",
        version=__version__,
        time=datetime.utcnow(),
    )


@app_router.post(
    "/apps",
    response_class=JSONResponse,
)
async def create_app(
    background_tasks: BackgroundTasks,
    create_app_request: Json = Form(...),
    upload_file: UploadFile = File(...),
) -> JSONResponse:
    _create_app_request = CreateAppRequest(**create_app_request)
    bundle_path = f"{_create_app_request.app_name}.tar.gz"
    with open(bundle_path, "wb") as buffer:
        buffer.write(upload_file.file.read())
    background_tasks.add_task(
        app_svc.unzip_and_run,
        bundle_path,
        _create_app_request.app_name,
    )
    return JSONResponse(
        status_code=200,
        content={"message": "App running in the cloud!"},
    )


@app_router.get(
    "/apps",
    response_model=GetAppsResponse,
)
async def get_apps(name: Optional[str] = None) -> GetAppsResponse:
    return app_svc.get_apps(name=name)


@app_router.delete(
    "/apps",
    response_model=DeleteAppsResponse,
)
async def delete_apps(
    bgt: BackgroundTasks, app_names: List[str] = Query(None)
) -> DeleteAppsResponse:
    bgt.add_task(app_svc.delete, app_names=app_names)
    return DeleteAppsResponse(deleted_apps=app_names)


api.include_router(health_router)
api.include_router(app_router)



from unittest import mock

from typer.testing import CliRunner

from hotbox.cli.main import app
from tests.mocks import MockEc2Service


@mock.patch(
    "hotbox.ec2.ec2_svc.ec2_client",
    return_value=MockEc2Service(),
)
async def test_create_ec2(
    mock_ec2_svc: mock.MagicMock,
    runner: CliRunner,
) -> None:
    result = runner.invoke(
        app,
        [
            "create",
            "ec2",
            "-f",
            '{"region": "us-east-1", "key_name": "fc-demo",'
            ' "security_group_ids": ["sg-0000"]}',
        ],
    )
    assert result.exit_code == 0
    assert result.output is not None


@mock.patch(
    "hotbox.ec2.ec2_svc.ec2_client",
    return_value=MockEc2Service(),
)
async def test_create_ec2_from_file(
    mock_ec2_svc: mock.MagicMock,
    runner: CliRunner,
) -> None:
    result = runner.invoke(
        app,
        [
            "create",
            "ec2",
            "-f",
            "./tests/assets/files/ec2.json",
        ],
    )
    assert result.exit_code == 0
    assert result.output is not None


@mock.patch(
    "hotbox.ec2.ec2_svc.ec2_client",
    return_value=MockEc2Service(),
)
async def test_create_ec2_from_stdin(
    mock_ec2_svc: mock.MagicMock,
    runner: CliRunner,
) -> None:
    result = runner.invoke(
        app=app,
        args=[
            "create",
            "ec2",
            "-f",
            "-",
        ],
        input='{"region": "us-east-1", "key_name": "fc-demo",'
        ' "security_group_ids": ["sg-0000"]}',
    )
    assert result.exit_code == 0
    assert result.output is not None


@mock.patch(
    "hotbox.ec2.ec2_svc.ec2_client",
    return_value=MockEc2Service(),
)
async def test_get_ec2(
    mock_ec2_svc: mock.MagicMock,
    runner: CliRunner,
) -> None:
    result = runner.invoke(
        app,
        [
            "get",
            "ec2",
            "--region",
            "us-east-1",
        ],
    )
    assert result.exit_code == 0
    assert result.output is not None


@mock.patch(
    "hotbox.ec2.ec2_svc.ec2_client",
    return_value=MockEc2Service(),
)
async def test_delete_ec2(
    mock_ec2_svc: mock.MagicMock,
    runner: CliRunner,
) -> None:
    result = runner.invoke(
        app,
        [
            "delete",
            "ec2",
            "i-0000",
            "--region",
            "us-east-1",
        ],
    )
    assert result.exit_code == 0
    assert result.output is not None


async def test_create_ec2_catch_no_file(
    runner: CliRunner,
) -> None:
    result = runner.invoke(
        app,
        [
            "create",
            "ec2",
            "-fcv",
            "v1.3.1",
        ],
    )
    assert result.exit_code == 1



import os
from typing import AsyncGenerator
from unittest import mock

from httpx import AsyncClient

from hotbox import __version__
from hotbox.const import API_V0


async def test_tz() -> None:
    assert os.environ["TZ"] == "UTC"


async def test_healthcheck(client: AsyncClient) -> None:
    response = await client.get(f"{API_V0}/healthcheck")
    assert response.status_code == 200
    assert response.json()["message"] == "ok"
    assert response.json()["version"] == __version__


@mock.patch(
    "hotbox.app.app_svc.unzip_and_run",
    return_value=None,
)
async def test_create_app(
    mock_unzip_and_run: mock.MagicMock, client: AsyncClient
) -> None:
    response = await client.post(
        f"{API_V0}/apps",
        files={
            "upload_file": (
                "test.tar.gz",
                b"test",
                "application/gzip",
            ),
            "create_app_request": (
                None,
                b'{"app_name": "test"}',
                "application/json",
            ),
        },
    )
    assert response.status_code == 200
    assert response.json()["message"] == "App running in the cloud!"
    os.remove("test.tar.gz")


async def test_get_apps_empty(client: AsyncClient) -> None:
    response = await client.get(f"{API_V0}/apps")
    assert response.status_code == 200
    assert response.json() == {"apps": {}}


async def test_get_apps_many(
    client: AsyncClient, create_test_fc_config_files: AsyncGenerator
) -> None:
    response = await client.get(f"{API_V0}/apps")
    data = response.json()
    assert response.status_code == 200
    assert len(data["apps"]) == 2
    assert set(data["apps"].keys()) == {"ui", "api"}


async def test_get_apps_on(
    client: AsyncClient, create_test_fc_config_files: AsyncGenerator
) -> None:
    response = await client.get(f"{API_V0}/apps?name=ui")
    data = response.json()
    assert response.status_code == 200
    assert len(data["apps"]) == 1
    assert set(data["apps"].keys()) == {"ui"}


async def test_delete_apps(
    client: AsyncClient, create_test_fc_config_files: AsyncGenerator
) -> None:
    response = await client.delete(f"{API_V0}/apps", params={"app_names": ["ui"]})
    assert response.status_code == 200
    assert response.json()["deleted_apps"] == ["ui"]
    response = await client.get(f"{API_V0}/apps")
    data = response.json()
    print(data)
    assert response.status_code == 200
    assert len(data["apps"]) == 1
    assert set(data["apps"].keys()) == {"api"}



from typing import Optional

import httpx
import orjson
from typer import Option, Typer, echo

from hotbox.ec2 import ec2_svc
from hotbox.settings import env
from hotbox.types import Routes

app = Typer(
    name="get",
    help="Get resources.",
    no_args_is_help=True,
)


@app.command(
    "ec2",
    help="Get ec2 resources.",
    no_args_is_help=True,
)
def get_ec2(
    region: str = Option(
        None,
        help="AWS Region.",
    ),
) -> None:
    response = ec2_svc.get(
        region=region,
    )
    echo(orjson.dumps(response))


@app.command(
    "app",
    help="Get apps.",
)
def get_apps(
    app_name: Optional[str] = Option(
        None,
        "-n",
        "--name",
        help="App name.",
    )
) -> None:
    response = httpx.get(
        url=env.HOTBOX_API_URL + Routes.apps,
    )
    echo(orjson.dumps(response.json()))



from typing import List

import orjson
from typer import Argument, Option, Typer, echo

from hotbox.app import app_svc
from hotbox.ec2 import ec2_svc

app = Typer(
    name="delete",
    help="Delete resources.",
    no_args_is_help=True,
)


@app.command(
    "ec2",
    help="Delete ec2 resources.",
    no_args_is_help=True,
)
def delete_ec2(
    ids: List[str] = Argument(
        ...,
        help="IDs of resources to delete.",
    ),
    region: str = Option(
        None,
        help="AWS Region.",
    ),
) -> None:
    response = ec2_svc.delete(ids, region)
    echo(orjson.dumps(response))


@app.command(
    "app",
    help="Delete apps.",
    no_args_is_help=True,
)
def delete_app(
    app_names: List[str] = Argument(
        ...,
        help="Names of apps to delete.",
    ),
) -> None:
    response = app_svc.make_delete_request(app_names=app_names)
    echo(f"Deleting apps: {','.join(response.json()['deleted_apps'])}")



from multiprocessing import Process

from typer.testing import CliRunner

from hotbox import __version__
from hotbox.cli.main import app


async def test_cli_version(runner: CliRunner) -> None:
    result = runner.invoke(app, ["version"])
    assert result.exit_code == 0
    assert result.output == __version__


async def test_cli_start_server(runner: CliRunner) -> None:
    p = Process(target=runner.invoke, args=(app, ["server"]))
    p.start()
    p.terminate()
    p.join()



import json
from unittest import mock

from httpx import Response
from typer.testing import CliRunner

from hotbox.cli.main import app


@mock.patch(
    "httpx.post",
    return_value=Response(
        status_code=200,
        json={"message": "ok"},
    ),
)
async def test_create_app_success(
    mock_upload_app_bundle_post: mock.MagicMock,
    runner: CliRunner,
) -> None:
    result = runner.invoke(
        app,
        [
            "create",
            "app",
            "-n",
            "test-app",
            "-c",
            "tests/assets/code/go",
        ],
    )
    assert result.exit_code == 0


@mock.patch(
    "httpx.post",
    return_value=Response(
        status_code=500,
        json={"message": "oh shit"},
    ),
)
async def test_create_app_fails(
    mock_upload_app_bundle_post: mock.MagicMock,
    runner: CliRunner,
) -> None:
    result = runner.invoke(
        app,
        [
            "create",
            "app",
            "-n",
            "test-app",
            "-c",
            "tests/assets/code/go",
        ],
    )
    assert result.exit_code == 1
    assert result.output == "Creating app!\n"
    assert result.stderr == (
        "Failed to upload app " "bundle with status code 500, " "and message: oh shit\n"
    )


@mock.patch(
    "httpx.post",
    return_value=Response(
        status_code=500,
        json={"message": "oh shit"},
    ),
)
async def test_create_app_fails_unsupported_lang(
    mock_upload_app_bundle_post: mock.MagicMock,
    runner: CliRunner,
) -> None:
    result = runner.invoke(
        app,
        [
            "create",
            "app",
            "-n",
            "test-app",
            "-c",
            "tests/assets/code/hoon",
        ],
    )
    assert result.exit_code == 1
    assert result.stdout == "Creating app!\n"
    assert result.stderr.startswith("Unsupported application content.")


@mock.patch("httpx.get", return_value=Response(status_code=200, json={"apps": []}))
async def test_get_apps(mock_get_response: mock.MagicMock, runner: CliRunner) -> None:
    result = runner.invoke(app, ["get", "app"])
    assert result.exit_code == 0
    assert json.loads(result.stdout.strip()) == {"apps": []}


@mock.patch(
    "httpx.delete",
    return_value=Response(status_code=200, json={"deleted_apps": ["my-app"]}),
)
async def test_delete_apps(
    mock_delete_response: mock.MagicMock, runner: CliRunner
) -> None:
    result = runner.invoke(app, ["delete", "app", "my-app"])
    assert result.exit_code == 0
    assert result.stdout == "Deleting apps: my-app\n"
